import os
import requests

from django.utils.translation import get_language

from wagtailaltgenerator.translation_providers import get_current_provider
from wagtailaltgenerator.providers import DescriptionResult


def get_original_rendition(image):
    """
    Returns the original image rendition. Needed to get the correct image URL for processing
    when using external storage (such as S3)
    """
    from wagtail.images.shortcuts import get_rendition_or_not_found
    return get_rendition_or_not_found(image, 'original')


def get_image_data(image_url):
    """
    Load external image and return byte data
    """
    image_data = requests.get(image_url)

    if image_data.status_code > 200 and image_data.status_code < 300:
        return None

    return image_data.content


def get_local_image_data(image_file):
    """
    Retrive byte data from a local file
    """
    abs_path = os.path.abspath(image_file.path)
    image_data = open(abs_path, "rb").read()
    return image_data


def translate_description_result(result):
    provider = get_current_provider()()

    lang_and_country_code = get_language()
    lang_code = lang_and_country_code.split("-")[0]

    strings = []
    if result.description:
        strings = [result.description]

    if result.tags:
        strings = [*strings, *result.tags]

    translated_strings = provider.translate(strings, target_language=lang_code)

    translated_description = (
        translated_strings[0] if result.description else result.description
    )

    translated_tags = (
        translated_strings[1:] if result.description else translated_strings
    )

    return DescriptionResult(description=translated_description, tags=translated_tags)



from django.conf import settings


DEFAULT_PROVIDER = "wagtailaltgenerator.providers.cognitive.Cognitive"  # NOQA

ALT_GENERATOR_USE_TAGS = getattr(settings, "ALT_GENERATOR_USE_TAGS", True)
ALT_GENERATOR_MAX_TAGS = getattr(settings, "ALT_GENERATOR_MAX_TAGS", -1)
ALT_GENERATOR_PROVIDER = getattr(settings, "ALT_GENERATOR_PROVIDER", DEFAULT_PROVIDER)
ALT_GENERATOR_MIN_CONFIDENCE = getattr(settings, "ALT_GENERATOR_MIN_CONFIDENCE", 50)
ALT_GENERATOR_PREFER_UPLOAD = getattr(settings, "ALT_GENERATOR_PREFER_UPLOAD", True)

ALT_GENERATOR_TRANSLATE_TO_LOCAL_LANG = getattr(
    settings, "ALT_GENERATOR_TRANSLATE_TO_LOCAL_LANG", None
)

ALT_GENERATOR_TRANSLATION_PROVIDER = getattr(
    settings,
    "ALT_GENERATOR_TRANSLATION_PROVIDER",
    "wagtailaltgenerator.translation_providers.google_translate.GoogleTranslate",
)


def get_setting(name):
    return getattr(settings, name, None) or globals()[name]



# -*- coding: utf-8 -*-
import logging

from django.conf import settings
import requests

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import AbstractProvider, DescriptionResult
from wagtailaltgenerator.utils import get_image_data, get_local_image_data, get_original_rendition


API_URL = "https://{}.api.cognitive.microsoft.com"

logger = logging.getLogger(__name__)


def describe_by_url(image_url):
    headers = {
        "Content-Type": "application/json",
        "Ocp-Apim-Subscription-Key": settings.COMPUTER_VISION_API_KEY,
    }

    json_data = {"url": image_url}

    endpoint = API_URL.format(settings.COMPUTER_VISION_REGION)
    response = requests.post(
        "{}{}".format(endpoint, "/vision/v1.0/describe"),
        headers=headers,
        json=json_data,
    )

    if response.status_code != 200:
        logger.warning([response, response.text])
        return None

    return response.json()


def describe_by_data(image_data):
    headers = {
        "Content-Type": "application/octet-stream",
        "Ocp-Apim-Subscription-Key": settings.COMPUTER_VISION_API_KEY,
    }

    endpoint = API_URL.format(settings.COMPUTER_VISION_REGION)
    response = requests.post(
        "{}{}".format(endpoint, "/vision/v1.0/describe"),
        data=image_data,
        headers=headers,
    )

    if response.status_code != 200:
        logger.warning([response, response.text])
        return None

    data = response.json()
    return data


class Cognitive(AbstractProvider):
    def describe(self, image):
        if app_settings.ALT_GENERATOR_PREFER_UPLOAD:
            if not image.is_stored_locally():
                rendition = get_original_rendition(image)
                image_data = get_image_data(rendition.url)
                data = describe_by_data(image_data)
            else:
                image_data = get_local_image_data(image.file)
                data = describe_by_data(image_data)
        else:
            rendition = get_original_rendition(image)
            data = describe_by_url(rendition.url)

        description = None
        tags = []

        min_confidence = float(app_settings.ALT_GENERATOR_MIN_CONFIDENCE) / 100.0

        if not data:
            return DescriptionResult(description=description, tags=tags)

        if "description" in data and len(data["description"]["captions"]):
            captions = data["description"]["captions"]
            captions = [
                caption["text"]
                for caption in captions
                if caption["confidence"] >= min_confidence
            ]

            if len(captions):
                description = captions[0]

        try:
            tags = data["description"]["tags"]
        except:  # NOQA
            pass

        return DescriptionResult(description=description, tags=tags)



from django.utils.module_loading import import_string

from wagtailaltgenerator import app_settings


def get_current_provider():
    provider = app_settings.ALT_GENERATOR_TRANSLATION_PROVIDER
    return get_provider(provider)


def get_provider(path):
    return import_string(path)


class AbstractTranslationProvider(object):
    def translate(strings, target_language, source_language="en"):
        raise NotImplementedError()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
wagtailaltgenerator
----------
Insert image description and tags with the help of computer vision
"""

__title__ = "wagtailaltgenerator"
__version__ = "4.1.2"
__build__ = 412
__author__ = "Martin Sandström"
__license__ = "MIT"
__copyright__ = "Copyright 2016-2020 Martin Sandström"

default_app_config = "wagtailaltgenerator.apps.AltGeneratorAppConfig"



import logging

from . import AbstractTranslationProvider


logger = logging.getLogger(__name__)


class GoogleTranslate(AbstractTranslationProvider):
    def translate(self, strings, target_language, source_language="en"):
        from google.cloud import translate

        client = translate.Client()
        response = client.translate(
            strings, source_language=source_language, target_language=target_language
        )

        return list(map(lambda x: x["translatedText"], response))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.conf import settings
from wagtail.images import get_image_model

from wagtailaltgenerator.providers import get_current_provider
from wagtailaltgenerator.utils import translate_description_result
from wagtailaltgenerator import app_settings
from wagtailaltgenerator.app_settings import get_setting


image_cls = get_image_model()


@receiver(
    post_save,
    sender=image_cls,
    dispatch_uid="wagtailaltgenerator.signals.apply_image_alt",
)
def apply_image_alt(sender, instance, **kwargs):
    # Only run on new images
    if not kwargs["created"]:
        return

    provider = get_current_provider()()
    image_url = instance.file.url

    result = provider.describe(instance)

    if get_setting("ALT_GENERATOR_TRANSLATE_TO_LOCAL_LANG"):
        result = translate_description_result(result)

    if image_url[-4:] == instance.title[-4:]:
        _apply_title(instance, result.description)

    if get_setting("ALT_GENERATOR_USE_TAGS"):
        _apply_tags(instance, result.tags)

    instance.save()


def _apply_title(instance, description):
    if not description:
        return

    instance.title = description


def _apply_tags(instance, tags):
    if not tags:
        return

    if app_settings.ALT_GENERATOR_MAX_TAGS != -1:
        tags = tags[: app_settings.ALT_GENERATOR_MAX_TAGS]

    instance.tags.add(*tags)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import re
from setuptools import setup, find_packages


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()


with open("README.md") as f:
    readme = f.read()

# Convert markdown to rst
try:
    from pypandoc import convert

    long_description = convert("README.md", "rst")
except:  # NOQA
    long_description = ""

version = ""
with open("wagtailaltgenerator/__init__.py", "r") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

setup(
    name="wagtailaltgenerator",
    version=version,
    description=(
        "Insert image description and tags with the help of computer vision"
    ),  # NOQA
    long_description=long_description,
    author="marteinn",
    author_email="martin@marteinn.se",
    url="https://github.com/marteinn/wagtail-alt-generator",
    packages=find_packages(
        exclude=("*.tests", "*.tests.*", "tests.*", "tests", "example*")
    ),
    include_package_data=True,
    install_requires=["requests", "wagtail>=1.12"],
    extras_require={
        "rekognition": ["boto3>=1.4,<1.5"],
        "google_vision": ["google-api-python-client>=1.5.5"],
        "google_translate": ["google-cloud-translate"],
    },
    license="MIT",
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "Framework :: Django",
        "Framework :: Wagtail",
        "Topic :: Utilities",
    ],
)



# -*- coding: utf-8 -*-
import base64

from googleapiclient import discovery
from oauth2client.client import GoogleCredentials

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import AbstractProvider, DescriptionResult
from wagtailaltgenerator.utils import get_image_data, get_local_image_data, get_original_rendition


class GoogleVision(AbstractProvider):
    def __init__(self, *args, **kwargs):
        credentials = GoogleCredentials.get_application_default()
        self.service = discovery.build("vision", "v1", credentials=credentials)
        super(GoogleVision, self).__init__(*args, **kwargs)

    def describe(self, image):
        if not image.is_stored_locally():
            rendition = get_original_rendition(image)
            image_data = get_image_data(rendition.url)
        else:
            image_data = get_local_image_data(image.file)

        description = None
        tags = []

        min_confidence = float(app_settings.ALT_GENERATOR_MIN_CONFIDENCE) / 100.0

        if image_data:
            image_data_base64 = base64.b64encode(image_data)

            max_results = app_settings.ALT_GENERATOR_MAX_TAGS

            request_body = {
                "requests": [
                    {
                        "image": {"content": image_data_base64.decode("UTF-8")},
                        "features": [{"type": "LABEL_DETECTION"}],
                    }
                ]
            }

            if max_results != -1:
                request_body["requests"][0]["features"][0][
                    "maxResults"
                ] = max_results  # NOQA

            service_request = self.service.images().annotate(body=request_body)

            response = service_request.execute()
            labels = response["responses"][0]["labelAnnotations"]
            tags = [
                label["description"]
                for label in labels
                if label["score"] >= min_confidence
            ]

        return DescriptionResult(description=description, tags=tags)



# -*- coding: utf-8 -*-

import boto3

from wagtailaltgenerator import app_settings
from wagtailaltgenerator.providers import AbstractProvider, DescriptionResult
from wagtailaltgenerator.utils import get_image_data, get_local_image_data, get_original_rendition


class Rekognition(AbstractProvider):
    def __init__(self, *args, **kwargs):
        self.client = boto3.client("rekognition")
        super(Rekognition, self).__init__(*args, **kwargs)

    def describe(self, image):
        if not image.is_stored_locally():
            rendition = get_original_rendition(image)
            image_data = get_image_data(rendition.url)
        else:
            image_data = get_local_image_data(image.file)

        description = None
        tags = []

        if image_data:
            response = self.client.detect_labels(
                Image={"Bytes": image_data},
                MinConfidence=app_settings.ALT_GENERATOR_MIN_CONFIDENCE,
            )
            if response["Labels"]:
                tags = [label["Name"] for label in response["Labels"]]

        return DescriptionResult(description=description, tags=tags)
